@@ -114,7 +114,8 @@ void transport_C(const refVecConst & mass_a, const refVecConst & mass_b,
     bool accuracy = false;
     trans_networkflow(mass_a, mass_b, cost_matrix, assignment, threads, accuracy, niter);
     which_nonzero(assignment, N, M, idx);
-  } else if (method == "sinkhorn" || method == "greenkhorn" || method == "randkhorn" || method == "gandkhorn") {
+  } else if (method == "sinkhorn" || method == "sinkhorn_log" ||
+    method == "greenkhorn" || method == "randkhorn" || method == "gandkhorn") {
     
     // void trans_approxOT(const refVecConst & mass_a, const refVecConst & mass_b, 
     //                     refMat cost_matrix, 
